@@ -71,7 +71,7 @@ struct GSManagerPrivate
         char        *logout_command;
         char        *keyboard_command;
 
-        char        *away_message;
+        char        *status_message;
 
         /* State */
         guint        active : 1;
@@ -111,7 +111,7 @@ enum {
         PROP_LOGOUT_TIMEOUT,
         PROP_LOGOUT_COMMAND,
         PROP_KEYBOARD_COMMAND,
-        PROP_AWAY_MESSAGE,
+        PROP_STATUS_MESSAGE,
         PROP_ACTIVE,
         PROP_THROTTLED,
 };
@@ -610,24 +610,24 @@ gs_manager_set_keyboard_command (GSManager  *manager,
         }
 
         for (l = manager->priv->windows; l; l = l->next) {
-                gs_window_set_logout_command (l->data, manager->priv->keyboard_command);
+                gs_window_set_keyboard_command (l->data, manager->priv->keyboard_command);
         }
 }
 
 void
-gs_manager_set_away_message (GSManager  *manager,
-                             const char *away_message)
+gs_manager_set_status_message (GSManager  *manager,
+                               const char *status_message)
 {
         GSList *l;
 
         g_return_if_fail (GS_IS_MANAGER (manager));
 
-        g_free (manager->priv->away_message);
+        g_free (manager->priv->status_message);
 
-        manager->priv->away_message = g_strdup (away_message);
+        manager->priv->status_message = g_strdup (status_message);
 
         for (l = manager->priv->windows; l; l = l->next) {
-                gs_window_set_away_message (l->data, manager->priv->away_message);
+                gs_window_set_status_message (l->data, manager->priv->status_message);
         }
 }
 
@@ -753,8 +753,8 @@ gs_manager_set_property (GObject            *object,
         case PROP_KEYBOARD_COMMAND:
                 gs_manager_set_keyboard_command (self, g_value_get_string (value));
                 break;
-        case PROP_AWAY_MESSAGE:
-                gs_manager_set_away_message (self, g_value_get_string (value));
+        case PROP_STATUS_MESSAGE:
+                gs_manager_set_status_message (self, g_value_get_string (value));
                 break;
         case PROP_CYCLE_TIMEOUT:
                 gs_manager_set_cycle_timeout (self, g_value_get_long (value));
@@ -803,8 +803,8 @@ gs_manager_get_property (GObject            *object,
         case PROP_KEYBOARD_COMMAND:
                 g_value_set_string (value, self->priv->keyboard_command);
                 break;
-        case PROP_AWAY_MESSAGE:
-                g_value_set_string (value, self->priv->away_message);
+        case PROP_STATUS_MESSAGE:
+                g_value_set_string (value, self->priv->status_message);
                 break;
         case PROP_CYCLE_TIMEOUT:
                 g_value_set_long (value, self->priv->cycle_timeout);
@@ -995,6 +995,7 @@ get_gconf_client (void)
         engine = gconf_engine_get_for_addresses (addresses, &error);
         if (engine == NULL) {
                 gs_debug ("Unable to get gconf engine for addresses: %s", error->message);
+                g_error_free (error);
         } else {
                 client = gconf_client_get_for_engine (engine);
         }
@@ -1444,11 +1445,15 @@ gs_manager_create_window_for_monitor (GSManager *manager,
         gs_window_set_logout_command (window, manager->priv->logout_command);
         gs_window_set_keyboard_enabled (window, manager->priv->keyboard_enabled);
         gs_window_set_keyboard_command (window, manager->priv->keyboard_command);
-        gs_window_set_away_message (window, manager->priv->away_message);
+        gs_window_set_status_message (window, manager->priv->status_message);
 
         connect_window_signals (manager, window);
 
         manager->priv->windows = g_slist_append (manager->priv->windows, window);
+
+        if (manager->priv->active && !manager->priv->fading) {
+                gtk_widget_show (window);
+        }
 }
 
 static void
@@ -1470,22 +1475,39 @@ on_screen_monitors_changed (GdkScreen *screen,
         if (n_monitors > n_windows) {
                 /* add more windows */
                 for (i = n_windows; i < n_monitors; i++) {
-                        gs_manager_create_window_for_monitor (manager, screen, i - 1);
+                        gs_manager_create_window_for_monitor (manager, screen, i);
                 }
         } else {
+
+                gdk_x11_grab_server ();
+
                 /* remove the extra windows */
-                for (l = manager->priv->windows; l != NULL; l = l->next) {
+                l = manager->priv->windows;
+                while (l != NULL) {
                         GdkScreen *this_screen;
                         int        this_monitor;
+                        GSList    *next = l->next;
 
                         this_screen = gs_window_get_screen (GS_WINDOW (l->data));
                         this_monitor = gs_window_get_monitor (GS_WINDOW (l->data));
                         if (this_screen == screen && this_monitor >= n_monitors) {
                                 manager_maybe_stop_job_for_window (manager, GS_WINDOW (l->data));
+                                g_hash_table_remove (manager->priv->jobs, l->data);
                                 gs_window_destroy (GS_WINDOW (l->data));
                                 manager->priv->windows = g_slist_delete_link (manager->priv->windows, l);
                         }
+                        l = next;
                 }
+
+                /* make sure there is a lock dialog on a connected monitor,
+                 * and that the keyboard is still properly grabbed after all
+                 * the windows above got destroyed*/
+                if (n_windows > n_monitors) {
+                        gs_manager_request_unlock (manager);
+                }
+
+                gdk_flush ();
+                gdk_x11_ungrab_server ();
         }
 }
 
@@ -1551,7 +1573,7 @@ gs_manager_finalize (GObject *object)
         free_themes (manager);
         g_free (manager->priv->logout_command);
         g_free (manager->priv->keyboard_command);
-        g_free (manager->priv->away_message);
+        g_free (manager->priv->status_message);
 
         remove_unfade_idle (manager);
         remove_timers (manager);
